////////////////////////////////////////////////////////////////////////////////
//
// CONFIG VARIABLES
//
///////////////////////////////////////////////////////////////////////////////////////////////////

var g_clocksToShow          = 3;       // maximum clocks displayed
var g_30min                 = 1800;    // refresh time in seconds (1800 * 1000 = 30 min)
var g_1hour                 = 3600;    // (3600 * 1000 = 60 min)                
var g_titleMaxLength        = 21;      // maximum title length when in sidebar
var g_titleMaxLengthUndock  = 52;      // maximum title length when undocked
var g_saveHtml              = "";
var g_saveNumber            = 0;
var g_activeLink            = true;
var clockSettings           = new Array();
var settingKeys             = new Array("Country", "Prov", "Name", "Key", "Latitude", "Longitude");
var tmOut                              // used for timeouts
var b_docked                           // store current docked state
var b_priorstate                       // store prior docked state
var b_needRefresh           = true;
///////////////////////////////////////////////////////////////////////////////////////////////////
//
// INITIALIZATION
//
////////////////////////////////////////////////////////////////////////////////
/// initMain()
///
/// First function which is called
function initMain()
{
    try
    {
        initGadget();
        b_priorstate = true;    // set initial state as docked
        checkState();   
    }
    catch(e)
    {
        alert("An a error ocurred: " + e);
    }   
   
}

/// initGadget()
///
/// Sets gadget data
function initGadget()
{
    // init settings
    System.Gadget.settingsUI = "settings.html";
    System.Gadget.Flyout.file = "flyout.html";     
    System.Gadget.onUndock = checkState;
    System.Gadget.onDock   = checkState;
    System.Gadget.visibilityChanged = checkVisibility;


    
}

System.Gadget.onSettingsClosed = function(event)
{
    if (event.closeAction == event.Action.commit)
    {
  		 // force a refresh of clocks
		 doRefresh();
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// CLOCK SETTINGS
//
////////////////////////////////////////////////////////////////////////////////

/// foreachSetting
///
/// Helper function which accepts a closure and applies it to all avaiable settings
function foreachSetting(closure)
{
    for(var settingIndex in settingKeys)
    {
       for (i=1; i<=3; i++){
	      var keyString = settingKeys[settingIndex] + i;
		  closure(keyString);
		  }
	}
}

/// loadClockSettings()
///
/// Load the user settings for each clock
function loadClockSettings()
{
    foreachSetting( function(name) { clockSettings[name] = readClockSetting(name); } );
}

/// readClockSetting()
///
/// Helper functions which reads a value from the gadget settings store,
/// or returns the default value when we use the gadget for the first time
function readClockSetting(name)
{
    var defaultValue = "";
    if(System.Gadget.Settings.readString(name)){
        return System.Gadget.Settings.readString(name);
    }else{
        return defaultValue;
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// CLOCK MANAGEMENT
//
////////////////////////////////////////////////////////////////////////////////
/// clearDisplay()
///
/// Clear the content of all html elements
function clearDisplay()
{
    for(var i = 1; i <= g_clocksToShow; i++)
    {
        var titleElement = document.getElementById("clock" + i + "_title");
        var gifElement = document.getElementById("clock" + i + "_html");
        titleElement.innerHTML = "";
        gifElement.innerHTML = "";
    }
}

/// displayClocks()
///
/// Write the clocks into the html elements
function displayClocks()
{
    for(var i = 1; i <= g_clocksToShow; i++)
    {
        // get elements
        var titleElement = document.getElementById("clock" + i + "_title");
        var gifElement = document.getElementById("clock" + i + "_html");
        var gifDisplay = "";
        //add 'random' parameter to prevent caching
        var dt = new Date();

        if (clockSettings["Name" + i] != "") {
           gifDisplay += "<a href=http://cleardarksky.com/c/" + clockSettings["Key" + i] + "key.html?1><img src=\"http://cleardarksky.com/c/";
           gifDisplay += clockSettings["Key" + i] + "cs0.gif?1?t=" + dt.getTime();
           if (System.Gadget.docked){
             gifDisplay += "\" border=\"0\" alt=\"Click to open site\" width=\"112\" height=\"13\"/></a>";
             }
           else {
             gifDisplay += "\" border=\"0\" alt=\"Click to open site\" width=\"265\" height=\"13\"/></a>";
             }
         }

        // limit length of clock name if necessary (only when docked) 
        var tmpClockName =  clockSettings["Name" + i];
        tmpClockName = tmpClockName.replace("_","");  // strip any trailing underscores
        clockTitle = tmpClockName; 
        
        if(tmpClockName != "")  // has been setup
        {
            
            clockTitle = clockTitle + ", " + clockSettings["Prov" + i];
        
            if ((System.Gadget.docked) && (clockTitle.length > g_titleMaxLength))
                clockTitle = clockTitle.substring(0,(g_titleMaxLength-3)) + "...";
        
            if ((!System.Gadget.docked) && (clockTitle.length > g_titleMaxLengthUndock))
                clockTitle = clockTitle.substring(0,(g_titleMaxLengthUndock-3)) + "...";
        }
            
        if (clockTitle == "") // not setup yet
        {
            clockTitle = "Click options to add a clock";
        }
        else   // set title as a link to the flyout
        { 
            clockTitle = "<a href=\"javascript:showMap(\'" + i + "\');\" title=\"Click to show map\">" + clockTitle + "</a>";
        }
                    
        // set content on gadget 
        try
        {                                   
           titleElement.innerHTML = clockTitle;
           gifElement.innerHTML = gifDisplay;
        }
        catch (e)
        {
           noConnection(e);  // display message 
        }
    }
    
}


/// This functions is called at the beginning and calls itself every g_clocksReload seconds
/// It reloads the clocks from the settings in case there are any changes
function doRefresh() 
{
    var d = new Date()
    var UTChour = d.getUTCHours();   // getUniversal Time Hour
    var currTime = d.toTimeString();  // get current local time
    clearTimeout(tmOut);             // clear prior timeout
    
    
    if(System.Gadget.visible)  // only refresh clocks if gadget is visible
    {
        // myalert("doRefresh called at " + currTime + " UTChour = " + UTChour); // debug
        loadClockSettings();
        displayClocks();
        b_needRefresh = false;
    }
    else
    {
        b_needRefresh = true;
    }
    // clocks are only refreshed during two 4 hour periods each day so we will only
    // use a 30 minute refresh rate during those periods based on Universal Time (UTC)
    // if time is between 12 - 15 EDT or 00 - 03 EDT then use a 30 minute refresh rate
    //   (time is between 16 - 19 UTC or 04 - 07 UTC)
    if ((UTChour >= 16 && UTChour <= 19) || (UTChour >= 04 && UTChour <= 07))
    {
        tmOut = setTimeout("doRefresh()",1000*g_30min);
    }
    else
    {
        var hrsToWait = 0;    // calculate hours to next UTC update window
        if (UTChour > 7 && UTChour < 16)
        {
            hrsToWait = 16 - UTChour;
            tmOut = setTimeout("doRefresh()",1000*g_1hour*hrsToWait);
        }
        else
        {
            if (UTChour >= 0 && UTChour < 4)
            {
                hrsToWait = 4 - UTChour;
                tmOut = setTimeout("doRefresh()",1000*g_1hour*hrsToWait);
            }
            else   // must be > 19 and <= 23
            {
                hrsToWait = 28 - UTChour;
                tmOut = setTimeout("doRefresh()",1000*g_1hour*hrsToWait);
            }
        }
    }
}

/// expand the gadget if it is undocked to show full size preview bars
///
function checkState() {
    if (System.Gadget.docked) {   
        document.body.style.width = "130px";
        System.Gadget.background = "images/bg_dock.png";
        var e = document.getElementById("clocklist");     
        e.style.height = "150px";
        e = document.getElementById("spacer");
        e.style.height = "13px";
        e.style.width = "130px";
        e = document.getElementById("clock1");
        e.setAttribute("className","itemLo");
        e = document.getElementById("clock2");
        e.setAttribute("className","itemLo");
        e = document.getElementById("clock3");
        e.setAttribute("className","itemLo");
    } else {
        document.body.style.width = "285px";
        System.Gadget.background = "images/bg_undock.png";
        var e = document.getElementById("clocklist");
        e.style.height = "150px";
        e = document.getElementById("spacer");
        e.style.height = "13px";
        e.style.width = "285px";
        e = document.getElementById("clock1");
        e.setAttribute("className","undocked");
        e = document.getElementById("clock2");
        e.setAttribute("className","undocked");
        e = document.getElementById("clock3");
        e.setAttribute("className","undocked");  
    }
    b_docked = System.Gadget.docked;    // store current state
    if(b_docked != b_priorstate)
    {
       doRefresh();   // only refresh if state changed
    }
    b_priorstate = b_docked;  // set prior state to current
}

// need to refresh if gadget has been invisible for a while
function checkVisibility()
{
    if(System.Gadget.visible)
    {
         if(b_needRefresh)  // clocks need a refresh
        {
 		    // force a refresh of clocks
		    doRefresh();
		}
    }
}



/// noConnection()
///
/// Clear the content of all html elements
function noConnection(e)
{
    for(var i = 1; i <= g_clocksToShow; i++)
    {
        var titleElement = document.getElementById("clock" + i + "_title");
        var gifElement = document.getElementById("clock" + i + "_html");
        if (i == 1)
            titleElement.innerText = "No Connection Available";
        else
            titleElement.innerText = "";
        gifElement.innerHTML = "";
    }
}

/// showMap(clockNumber)
///
/// Open the flyout to display location map
function showMap(clockNumber)
{
    var titleElement = document.getElementById("clock" + clockNumber + "_title");
    g_saveHtml = titleElement.innerHTML;
    g_saveNumber = clockNumber;
    
    if (clockSettings["Latitude" + clockNumber] == 0)  // private site
    {
        titleElement.innerHTML = "<span style=\"color: Red; background-color: White\">* PRIVATE LOCATION *</span>";   // user message
        setTimeout("resetTitle(g_activeLink)",3000);  // reset clock title after three seconds
    }
    else  // users may see location on map
    {
        if (System.Gadget.Flyout.show==true)
        {
            System.Gadget.Flyout.show=false;   // close Flyout if already open
        }
        else
        {
            titleElement.innerHTML = "<span style=\"color: Yellow\">Loading Map Data...</span>";   // wait message
            
            System.Gadget.Settings.writeString("showMapNumber", clockNumber);  // store selected clock number

            System.Gadget.Flyout.show=true;     // activate flyout
        }
        
        System.Gadget.Flyout.onShow = function()
        {
            resetTitle(false);  // put back clock title
        }
        
        System.Gadget.Flyout.onHide = function()
        {
            resetTitle(true);  // put back clock title
        }
    }
}

function resetTitle(activeLink)
{

    var titleElement = document.getElementById("clock" + g_saveNumber + "_title");
    if (activeLink)   // restore live link clock title
    {
        titleElement.innerHTML = g_saveHtml;  
    }
    else  // restore text portion of clock title
    {
        titleElement.innerHTML = g_saveHtml.substring(61,(g_saveHtml.length-4));  
    }
}

// for debug use only
function myalert(msg)
{
    var oShell = new ActiveXObject("WScript.Shell");
    oShell.Popup(msg);
}

// end of gadget.js script functions