////////////////////////////////////////////////////////////////////////////////
//
// VARIABLES
//
//////////////////////////////////////////////////////////////////////////////////////////////////////
var tmpclockSettings           = new Array();
var settingKeys                = new Array("Country", "Prov", "Name", "Key", "Latitude", "Longitude");
var clockCount                 = 0;
//////////////////////////////////////////////////////////////////////////////////////////////////////
//
// INITIALIZATION
//
////////////////////////////////////////////////////////////////////////////////

/// initSettings()
/// 
/// Is called whenever the settings dialog is opened and prepares the form.
function initSettings()
{
	// Add an event listener to the close event
	System.Gadget.onSettingsClosing = settingsClosing;
	// load clocks array from XML data
	loadClocks();
	// load current settings
	loadClockSettings();
	
	// If no clock has been setup for a site then disable the ProvinceState 
	// and Clock selection boxes until Country chosen
	for (var i=1; i<=3; i++)
	{
	    var clockTest = tmpclockSettings["Name" + i];
	    document.GadgetSettings["C" + i].disabled=false;  // enable country select
        if (clockTest == "")
        {
            document.GadgetSettings["P" + i].disabled=true;
            document.GadgetSettings["N" + i].disabled=true;
        }
        else  // we need to show the current settings
        {
            var tmpVal = tmpclockSettings["Country" + i];
            // set country index to tmpVal
            var optList=document.GadgetSettings["C" + i];
            SetOptionInList(optList,tmpVal); 
            var tmpBox = "P" + i;
            loadProvStateOptionsBox(tmpVal,tmpBox);
            // set province index to match tmpVal
            var optList=document.GadgetSettings["P" + i];
            tmpVal = tmpclockSettings["Prov" + i];
            SetOptionInList(optList,tmpVal); 
            tmpBox = "N" + i;
            loadClockOptionsBox(tmpVal,tmpBox)
            tmpVal = tmpclockSettings["Name" + i];
            // set clock index to match tmpVal
            var optList=document.GadgetSettings["N" + i];
            SetOptionInList(optList,tmpVal);
        }
	}
	// clear loading message and show column titles
	var title1Element = document.getElementById("title1");
	var title2Element = document.getElementById("title2");
	var msgElement = document.getElementById("message1");
    title1Element.innerHTML = "COUNTRY";
	title2Element.innerHTML = "PROVINCE or STATE /<br />CLOCK NAME";
	// enable the refresh button
	document.GadgetSettings["RefreshButton"].disabled=false;
	// show the number of clocks in the database
	for (clockName in clockArray)
   {
        clockCount++;
   }
	msgElement.innerHTML = clockCount + " Clocks Installed";
}

/// settingsClosing(event)
///
/// Event handler for the onSettingsClosing Event.
function settingsClosing(event)
{
	// Settings Dialog ended with "ok"
	if(event.closeAction == event.Action.commit)
	{
		// Save new settings
		updateSettings();
	}
	// Settings Dialog ended with "Cancel"
	else if (event.closeAction == event.Action.cancel)
	{
        // do nothing
	}

	// Do not propagate the event
	event.cancel = false;
}

/// foreachSetting
///
/// Helper function which accepts a closure and applies it to all avaiable settings
function foreachSetting(closure)
{
    for(var settingIndex in settingKeys)
    {
       for (i=1; i<=3; i++){
	      var keyString = settingKeys[settingIndex] + i;
		  closure(keyString);
		  }
	}
}

/// loadClockSettings()
///
/// Load the user settings for each clock
function loadClockSettings()
{
    foreachSetting( function(name) { tmpclockSettings[name] = readClockSetting(name); } );
}

/// readClockSetting()
///
/// Helper functions which reads a value from the gadget settings store,
/// or returns the default value when we use the gadget for the first time
function readClockSetting(name)
{
    var defaultValue = "";
    if(System.Gadget.Settings.readString(name)){
        return System.Gadget.Settings.readString(name);
    }else{
        return defaultValue;
    }
}


/// updateSettings()
///
/// Function to get and store clock preferences selected.
function updateSettings()
{
	// Iterate through all our settings
	foreachSetting( function(name) { saveSettings(name); } );
}

/// saveSetting(name of setting)
///
/// Store information on Country, ProvState, Name, URL and GIF
function saveSettings(name)
{
	// Save the values stored in temp array
	if(tmpclockSettings[name] != "")
		System.Gadget.Settings.writeString(name, tmpclockSettings[name]);
	else
		System.Gadget.Settings.writeString(name, "");
}


/// loadProvStateOptionsBox(country,selectBox)
///
/// Load the ProvState entries for a selected country
function loadProvStateOptionsBox(country,selectBox)
{
 switch(country)
 {
   	case "Canada":
   	    loadProvStateList(canadaArray,selectBox); 
        break;
    
   	case "USA":    	
   	    loadProvStateList(usaArray,selectBox); 
        break;
	
   	case "Mexico":    	
   	    loadProvStateList(mexicoArray,selectBox); 
        break;
        
    case "Bahamas":    	
   	    loadProvStateList(bahamasArray,selectBox); 
        break;
 }
}

/// loadProvStateList(listArray,selectBox)
///
/// Load the ProvState entries for a selected country
function loadProvStateList(listArray,selectBox)
{
 var master=document.GadgetSettings[selectBox];
 // make sure list is sorted
 listArray.sort(); 
 // clear the options list
 master.options.length=0;
 // first entry must be select text to force a change
 var selectText = "Select Province or State";
 master.options[master.options.length]=new Option(selectText,selectText);
 // populate names and values
 for (i=0; i<listArray.length; i++)
 {
    master.options[master.options.length]=new Option(listArray[i],listArray[i]);
 }
 // enable the selection box
 master.options.disabled=false;
}

//Function alerts the index of the selected option within form
function alertselected(selectBox,value){

  if(selectBox.charAt(0) == 'P')  // if country selected
  {
     clearClockList(selectBox);                 // clear and disable clock list
     loadProvStateOptionsBox(value,selectBox);  // load country provstate options
  }
  if(selectBox.charAt(0) == 'N')  // if province-state selected
  {
     loadClockOptionsBox(value,selectBox);  // load associated clock name options
  }
  if(selectBox.charAt(0) == 'S')  // if clock selected
  {
     storeClockSettings(value,selectBox.charAt(1));  // store clock info in tmp array
  }
}

/// storeClockSettings(clockName,index)
///
/// Load the clock info for a specific site and clock name
function storeClockSettings(clockName,index)
{
    var oClock = clockArray[clockName];      // get clock object
    tmpclockSettings["Country" + index] = oClock.country;
    tmpclockSettings["Prov" + index] = oClock.provstate;
    tmpclockSettings["Name" + index] = clockName;
    tmpclockSettings["Key" + index] = oClock.key;
    tmpclockSettings["Latitude" + index] = oClock.latitude; 
    tmpclockSettings["Longitude" + index] = oClock.longitude;  
}

/// loadClockOptionsBox(provstate,selectBox)
///
/// Load the clock name entries for a selected province or state
function loadClockOptionsBox(provstate,selectBox)
{
   var master=document.GadgetSettings[selectBox];
   // clear the options list
   master.options.length=0;
   // first entry must be select text to force a change
   var selectText = "Select Clock";
   master.options[master.options.length]=new Option(selectText,selectText);
   var i = 0;
   var listArray = new Array();
   for (clockName in clockArray)
   {
       // populate names and values if provstate matches clock record
      if(clockArray[clockName].provstate == provstate)
      {
         listArray[i] = clockName;
         i++;
      }
 
   }
   // make sure list is sorted
   listArray.sort();  
   // populate clock names and values
   var displayName = "";
   for (i=0; i<listArray.length; i++)
     {
        displayName = listArray[i].replace("_","");  // strip any trailing underscores
        master.options[master.options.length]=new Option(displayName,listArray[i]);
     }
    // enable the selection box
    master.options.disabled=false;
}

/// clearClockList()
///
/// Clear the associated clock entries when new country selected
function clearClockList(selectBox)
{
     var master=document.GadgetSettings["N" + selectBox.charAt(1)]; 
     // clear the options list
     master.options.length=0;
     // first entry must be select text to force a change
     var selectText = "Select Clock";
     master.options[master.options.length]=new Option(selectText,selectText);
     // disable the selection box
     master.options.disabled=true;
}

//  SetOptionInList()
//  Set the selected option to match the current clock setting
//                  
function SetOptionInList(lstSelectList,strValue)
{
          var intIndex = 0;
          // Loop through all the options
          for( intIndex = 0; intIndex < lstSelectList.options.length; intIndex++ )
          {
                // Is this the ID we are looking for?
                if( lstSelectList.options[intIndex].value == strValue ) 
                {
                      // Select it
                      lstSelectList.selectedIndex = intIndex;
                      // stop searching
                      break;
                }
          }
}

//  BuildDatabase()
//  Retrieve the latest clock data and update the XML database
function BuildDatabase()
{
    document.GadgetSettings["RefreshButton"].value="Checking...";
    document.GadgetSettings["RefreshButton"].disabled=true;
    var msgElement = document.getElementById("message1");
    var iconElement = document.getElementById("picture1");
    var TEXT_FILE1_URL = 'http://cleardarksky.com/t/chart_keys00.txt';
    var TEXT_FILE2_URL = 'http://cleardarksky.com/t/chart_prop00.txt';
    var keysFile = System.Gadget.path + "\\data\\chart_keys00.txt";
    var propFile = System.Gadget.path + "\\data\\chart_prop00.txt";
    var updtProg = System.Gadget.path + "\\bin\\CSCgadgetUpdate.exe";
    var OKtoProceed = 1;
    var fs = new ActiveXObject("Scripting.FileSystemObject");
    var txtFile = new ActiveXObject("microsoft.xmlhttp");
    
    for (i=1; i<=2; i++)   
    {
        if(i == 1)
            {
            var url = TEXT_FILE1_URL;
            var tmpFile = keysFile; 
            }
        else
            {
            var url = TEXT_FILE2_URL;
            var tmpFile = propFile;
            }
        try {
            var clockdata = ""; 
            txtFile.open("GET",url,false);
            txtFile.setRequestHeader("Content-Type","text/plain")
            txtFile.onreadystatechange=function() {
                if (txtFile.readyState==4)
                 {
                    if (txtFile.status==200)
                    { 
                        if(i == 1)
                            clockdata = txtFile.responseText;
                        else
                            clockdata = txtFile.responseBody;  // required for extended ASCII names
                    } 
                    else
                    {
                    //myalert("Error: Unable to download " + url);
                    OKtoProceed = 0;
                    }
                 }
            }
            txtFile.send(); 
            }
        catch (e) {
            // unable to download file
            // myalert("Unable to download " + url);
            OKtoProceed = 0;
            }
        var a = null;
        var b = null;
        try {
            if(i == 1)
                {      
                a = fs.CreateTextFile(tmpFile, true, false);
                a.Write(clockdata);  
                a.Close();
                a = null;
                }
            else
                { 
                b = fs.CreateTextFile(tmpFile, true, true); // set Unicode to true
                b.Write(clockdata);  // must strip first two FF FE bytes later
                b.Close();
                b = null;
                }
            } // end if
        catch (e) {
            // unable to download file
            //myalert("Unable to write " + tmpFile);
            //myalert(e.name + ": " + e.message);
            OKtoProceed = 0;
            if (a != null)   // close files if still open
               a.close();
            if (b != null)
               b.close();
            }
   } // next i
   
   // OKtoProceed = 0;  // debug

   if(OKtoProceed)  // ony update if both files received okay
   {
   	   // show processing message
   	   document.GadgetSettings["RefreshButton"].value="Updating...";
   	   msgElement.innerHTML = "Files received, launching database update process&hellip;";
   	   iconElement.innerHTML = "<img src=\"./images/info.png\">";

       // run program to update XML database from new downloaded files
       System.Shell.execute(updtProg, System.Gadget.path);
       
       // clear processing message
       msgElement.innerHTML = "";
       iconElement.innerHTML = "";
   }
   else
   {
       // show error message
       document.GadgetSettings["RefreshButton"].value="No Data";
       msgElement.innerHTML = "Service unavailable, please try later.";
       iconElement.innerHTML = "<img src=\"./images/info.png\">";
   }
   document.GadgetSettings["RefreshButton"].value="Check for updates";
     
}

function pauseMiliSecs(millis) 
{
var date = new Date();
var curDate = null;

do { curDate = new Date(); } 
while(curDate-date < millis);
} 



// DEBUG:
function myalert(msg)
{
    var oShell = new ActiveXObject("WScript.Shell");
    oShell.Popup(msg);
}

// end of settings.js script functions 