////////////////////////////////////////////////////////////////
//	File: xml.js
//
//	Summary: contains methods used to parse XML for clock array
////////////////////////////////////////////////////////////////

////////////////////////////////////////////
//  GLOLBAL XML VARIABLES
////////////////////////////////////////////
var gXmlDoc;
var gSelectedNode;
var clockArray = new Array();    // array to store clock objects
var canadaArray = new Array();   // arrays to store province state sublists
var mexicoArray = new Array();
var usaArray = new Array();
var bahamasArray = new Array();


function clockObj()
{
    var o = new Object();
    // Create a new empty clock object
    // Clock Name will be used to store data in array
    o.country = "";
    o.provstate = "";
    o.key = "";
    o.latitude = "";
    o.longitude = "";
    return o;
    // Return the object
}


////////////////////////////////////////////////
//  loadClocks()
//
//  Summary: loads clock records from an Xml
//      file to an multi-dimension array
/////////////////////////////////////////////////
function loadClocks()
{
	//check to see if we have a ref to the Xml document
    if(gXmlDoc == null)
    {
        //get the path to the Xml document
        var path = ".\\data\\CSCtable.xml";
        
        //load the Xml doc from disk
        gXmlDoc = new ActiveXObject("Microsoft.XMLDOM");
        gXmlDoc.async = false;
        gXmlDoc.load(path);
    }
    
    //get the root element of the document
	var root = gXmlDoc.getElementsByTagName('CSCTable').item(0);
	
	//load the individual clock strings
	for(i = 0; i < root.childNodes.length; i++)
	{
	    //get the data node
		var node = root.childNodes.item(i);
		
		//get data strings for each clock
        var country = node.childNodes.item(0).text;
		var provstate = node.childNodes.item(1).text;
		var name = node.childNodes.item(2).text;
		var key = node.childNodes.item(3).text;
		var latitude = node.childNodes.item(4).text;
		var longitude = node.childNodes.item(5).text;
		var tryagain = 1;
		
		//place data in array
		
		myClock = new clockObj();	// Create new instance
		myClock.country = country;
        myClock.provstate = provstate;
        myClock.key = key;
        myClock.latitude = latitude;
        myClock.longitude = longitude;
        while(tryagain == 1)
        {
            if (clockArray[name] != null)  // clock with same name exists
            {
                name = name + "_";   // append an underscore
            }
            else
            {
                tryagain = 0;
            }
        }
        
        
        clockArray[name] = myClock;  // store object in array
        
        
        // build sublists of provstate by country
       	switch(country)
       	{
       	case "Canada":
       	    loadProvState(canadaArray,provstate); 
            break;
        
       	case "USA":    	
       	    loadProvState(usaArray,provstate); 
            break;

		
       	case "Mexico":    	
       	    loadProvState(mexicoArray,provstate); 
            break;
            
        case "Bahamas":    	
       	    loadProvState(bahamasArray,provstate); 
            break;
            
		}  // end of select	cases
			
	}  // end of load clocks for loop
	
} // end of function

/////////////////////////////////////////////////////////////////////////
//  loadProvState(psArray,pstate)
//  Parameters: psArray - country array that contains ProvState strings
//              pstate - provincestate string to be loaded
//  Summary: builds sublist array of Province or State by country
/////////////////////////////////////////////////////////////////////////
function loadProvState(psArray,pstate)
{
    found = false;
    for (j=0; psArray[j] != null; j++){
       if (psArray[j] == pstate){
          found = true;
          break;
          }
       }
    if (!found){
       // add new province or state to array
       psArray[j] = pstate;
       }		
}

// end of xml.js script functions